<?php

App::uses('AppComponent', 'Controller/Component');

class CategoryOverridesComponent extends AppComponent {

/**
 * Override the content and meta data of a category based on options selected in a filter.
 *
 * @param array $data The category data to override.
 * @return array Overridden data.
 */
	public function override($data) {
		if (empty(Configure::read('EvShop.useCategoryOverrides'))) {
			return $data;
		}

		if (empty($this->_controller->Category)) {
			$this->_controller->loadModel('EvShop.Category');
		}

		$Category = $this->_controller->Category;

		if (empty($data[$Category->alias]['id'])) {
			return $data;
		}

		$brand = $this->_getFilterBrand();

		$options = $this->_getFilterOptions();

		//If there are no options to override on then return the current data.
		if (empty($options) && (empty($brand) || is_array($brand))) {
			return $data;
		}

		if (empty($this->_controller->CategoryOverride)) {
			$this->_controller->loadModel('EvShop.CategoryOverride');
		}

		$override = $this->_controller->CategoryOverride->findOverride(
			$data[$Category->alias]['id'],
			$brand,
			$options
		);

		if (empty($override)) {
			return $data;
		}

		$data = $this->_mergeOverride($data, $override);

		return $data;
	}

/**
 * Get the id of the brand from the filter. Overrides can only be associated with a single brand so if multiple brands
 * have been selected in the filter then an override can't be applied.
 *
 * As filters can be different per project, extend to apply an override.
 *
 * @return int The id of the brand to override by.
 */
	protected function _getFilterBrand() {
		return 0;
	}

/**
 * Get a list of options from the filter.
 *
 * As filters can be different per project, extend to apply an override.
 *
 * @return array List of options.
 */
	protected function _getFilterOptions() {
		return [];
	}

/**
 * Merge the override data into the category data. If meta fields have been overridden then merge them into the meta
 * data fields.
 *
 * @param array $data     The category data to override
 * @param array $override The data to override with.
 * @return array The overridden category data.
 */
	protected function _mergeOverride($data, $override) {
		foreach ($override['CategoryOverride'] as $field => $value) {
			if (empty($value)) {
				//If the override value hasn't been set then don't override the original data.
				continue;
			}

			$alias = 'Category';

			if (strpos($field, 'meta_') === 0) {
				//This is a meta field
				$alias = 'MetaData';
				$field = preg_replace('/meta_/', '', $field, 1);
			}

			$data[$alias][$field] = $value;
		}

		return $data;
	}
}
